/******************************************************************************
 *
 * FILE: sar_adc.c
 *
 * Description:
 *    SAR ADC conversion
 *
 * History:
 *    17.11.2017   PN   Initial version
 *    08.02.2018   MF   Update for SPC574K72
 *
 ******************************************************************************
 * Copyright (C) 2015-2017 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

/* BSP support package */
#include "bsp.h"
/* SARADC example header */
#include "saradc.h"


/* ============================================================================
 *            GLOBAL VARIABLES
 * ==========================================================================*/

uint32_t g_saradc_result_uint = 0;
float 	g_saradc_result_volts = 0;

static uint32_t saradc_GetData (uint32_t channel);

/* ============================================================================
 *            ISR FUNCTIONS
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * FUNCTION: SaradcbIsrHandler()
 *    SAR ADC B End Of Conversion ISR Handler
 * ----------------------------------------------------------------------------
 */

void SaradcbIsrHandler(void)
{
	g_saradc_result_uint = saradc_GetData(0);/* read data from channel ANP[0]*/
	g_saradc_result_volts = (saradc_GetData(0) * 1.22)/1000;/* calculate input
	                                                           voltage level */
	SARADC_B.ISR.R = 0x00000003;			  /* Clear ANP[0] interrupt flag */
}


/* ============================================================================
 *            ADC example API
 * ==========================================================================*/
/* ----------------------------------------------------------------------------
 * Name:        saradc_Init
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description: Initialization of SAR ADC - Module, Interrupt, IRQ
 *				Initialization of PIN for analog function
 * --------------------------------------------------------------------------*/

void saradc_Init (void)
{
	SARADC_B.MCR.B.NTRGEN = 1; /* Normal trigger enabled to start a normal
	                              conversion */
	SARADC_B.MCR.B.MODE = 0x0; /* Sets conversion mode on SARADCB, single
	                              conversion */
 	SARADC_B.MCR.B.OWREN = 1; /* SAR ADC B overwrite enable set */
	SARADC_B.SARADC_ICIMR[0].R = 0x00000001U; /* Interrupt of ANP[0] enabled */
	SARADC_B.IMR.B.MSKNEOC = 1U; /* NEOC interrupt is enabled */
	SARADC_B.CTR[0].B.CRES = SARADC_CTR0_CRES;
	SARADC_B.CTR[0].B.INPSAMP = SARADC_B_CTR0_INPSAMP;
	SARADC_B.ICNCMR[0].R = 0x00000001U; /* Sets channel ANP[0] to be converted
	                                       in SARADCB 0 (PB[0]) */
	SARADC_B.ICDR[0].R = 0x0U; /* Setup internal channel  for SARADCB,
	                              no pre-charge */

	/* Register interrupt handler */
	bsp_isr_RegisterHandler(UC_SARADCB_ISR, 10, SaradcbIsrHandler);
	/* Register interrupt handler, priority 10 */

	SARADC_B.MCR.B.PWDN = 0; /* Power up SAR ADC B */

	/* Configure SAR ADC pins */
    SIUL2.MSCR_IO[SARADC_PIN_PB0].B.SSS = 0; 	/* ADCSAR_DIG_B - PB[0] ANP[0]*/
    SIUL2.MSCR_IO[SARADC_PIN_PB0].B.APC = 1;   /* Analog Pad Control  Enable*/
}

void saradc_StartScan (void)
{   
	SARADC_B.MCR.B.NSTART = 0x1;                /* Starts CSCAN onSARADCB */
}   

static uint32_t saradc_GetData (uint32_t channel)
{
	return SARADC_B.ICDR[channel].B.CDATA; /* Read SARADCB conversion result */
}


